@@ -1,4 +1,4 @@
-# Copyright (c) 1997-2000 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 1997-2004 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
@@ -27,7 +27,7 @@ use Net::LDAP::Constant qw(LDAP_SUCCESS
 			   LDAP_UNAVAILABLE
 			);
 
-$VERSION 	= "0.30";
+$VERSION 	= "0.32";
 @ISA     	= qw(Tie::StdHash Net::LDAP::Extra);
 $LDAP_VERSION 	= 3;      # default LDAP protocol version
 
@@ -135,6 +135,7 @@ sub connect_ldap {
   $ldap->{net_ldap_socket} = IO::Socket::INET->new(
     PeerAddr   => $host,
     PeerPort   => $arg->{port} || '389',
+    LocalAddr  => $arg->{localaddr} || undef,
     Proto      => 'tcp',
     MultiHomed => $arg->{multihomed},
     Timeout    => defined $arg->{timeout}
@@ -156,6 +157,7 @@ sub connect_ldaps {
   $ldap->{'net_ldap_socket'} = IO::Socket::SSL->new(
     PeerAddr 	    => $host,
     PeerPort 	    => $arg->{'port'} || '636',
+    LocalAddr       => $arg->{localaddr} || undef,
     Proto    	    => 'tcp',
     Timeout  	    => defined $arg->{'timeout'} ? $arg->{'timeout'} : 120,
     _SSL_context_init_args($arg)
@@ -328,10 +330,6 @@ sub bind {
       if $ldap->{net_ldap_version} < 3;
 
     my $sasl = $passwd;
-    # Tell the SASL object our user identifier
-    $sasl->callback( user => "dn: $stash{name}")
-      unless $sasl->callback('user');
-
     my $sasl_conn = $sasl->client_new("ldap",$ldap->{net_ldap_host});
 
     # Tell SASL the local and server IP addresses
@@ -721,7 +719,8 @@ sub _sendmesg {
       if $debug & 4;
   }
 
-  my $socket = $ldap->socket or return LDAP_SERVER_DOWN;
+  my $socket = $ldap->socket
+      or return _error($ldap, $mesg, LDAP_SERVER_DOWN, "$!");
 
   syswrite($socket, $mesg->pdu, length($mesg->pdu))
     or return _error($ldap, $mesg, LDAP_LOCAL_ERROR,"$!");
@@ -888,6 +887,7 @@ sub root_dse {
 		  altServer
 		  supportedExtension
 		  supportedControl
+		  supportedFeatures
 		  supportedSASLMechanisms
 		  supportedLDAPVersion
 		)];
